@@ -34,7 +34,9 @@ int main(int argc, const char **argv)
   argv_to_vec(argc, argv, args);
   env_to_vec(args);
   DEFINE_CONF_VARS(usage);
-  common_init(args, "cauthtool", false, false);
+
+  common_set_defaults(false);
+  common_init(args, "cauthtool", false);
 
   const char *me = argv[0];
 
