@@ -12,8 +12,8 @@
  * 
  */
 
-#ifndef __CEPH_CONFIG_H
-#define __CEPH_CONFIG_H
+#ifndef CEPH_CONFIG_H
+#define CEPH_CONFIG_H
 
 extern struct ceph_file_layout g_default_file_layout;
 
@@ -45,6 +45,7 @@ struct md_config_t {
   char *id;
   char *name;
   char *alt_name;
+  char *host;
 
   int num_mon;
   int num_mds;
@@ -64,9 +65,12 @@ struct md_config_t {
   const char *logger_subdir;
   const char *logger_dir;
 
+  const char *log_file;
   const char *log_dir;
   const char *log_sym_dir;
+  int log_sym_history;
   bool log_to_stdout;
+  bool log_per_instance;
 
   const char *pid_file;
 
@@ -116,7 +120,6 @@ struct md_config_t {
   char *key;
   char *keyfile;
   char *keyring;
-  char *supported_auth;
 
   // messenger
 
@@ -135,6 +138,9 @@ struct md_config_t {
   bool ms_die_on_failure;
   bool ms_nocrc;
   bool ms_die_on_bad_msg;
+  uint64_t ms_dispatch_throttle_bytes;
+  bool ms_bind_ipv6;
+  uint64_t ms_rwthread_stack_bytes;
 
   // mon
   const char *mon_data;
@@ -143,6 +149,7 @@ struct md_config_t {
   int mon_osd_down_out_interval;
   float mon_lease;
   float mon_lease_renew_interval;
+  float mon_lease_wiggle_room;
   float mon_lease_ack_timeout;
   float mon_lease_timeout;
   float mon_accept_timeout;
@@ -157,11 +164,15 @@ struct md_config_t {
   double paxos_observer_timeout;
 
   // auth
+  char *auth_supported;
   double auth_mon_ticket_ttl;
   double auth_service_ticket_ttl;
   int auth_nonce_len;
   EntityName *entity_name;
 
+  double mon_client_hunt_interval;
+  double mon_client_ping_interval;
+
   // client
   int      client_cache_size;
   float    client_cache_mid;
@@ -177,6 +188,7 @@ struct md_config_t {
   long long client_readahead_max_bytes;
   long long client_readahead_max_periods;
   const char *client_snapdir;
+  const char *client_mountpoint;
   int fuse_direct_io;
   bool fuse_ll;
 
@@ -262,6 +274,9 @@ struct md_config_t {
   float mds_bal_midchunk;
   float mds_bal_minchunk;
 
+  int mds_bal_target_removal_min;
+  int mds_bal_target_removal_max;
+
   bool  mds_trim_on_rejoin;
   int   mds_shutdown_check;
 
@@ -295,6 +310,8 @@ struct md_config_t {
   double osd_shed_reads_min_latency_diff;
   double osd_shed_reads_min_latency_ratio;
 
+  uint64_t osd_client_message_size_cap;
+
   bool  osd_immediate_read_from_cache;
   bool  osd_exclusive_caching;
   double osd_stat_refresh_interval;
@@ -309,6 +326,12 @@ struct md_config_t {
   int   osd_max_rep;
   int   osd_min_raid_width;
   int   osd_max_raid_width;
+
+  int osd_pool_default_crush_rule;
+  int osd_pool_default_size;
+  int osd_pool_default_pg_num;
+  int osd_pool_default_pgp_num;
+
   int   osd_op_threads;
   int   osd_max_opq;
   int   osd_disk_threads;
@@ -326,10 +349,14 @@ struct md_config_t {
 
   float osd_recovery_delay_start;
   int osd_recovery_max_active;
+  uint64_t osd_recovery_max_chunk;
 
   bool osd_auto_weight;
 
   bool osd_class_timeout;
+  const char *osd_class_tmp;
+
+  bool osd_check_for_log_corruption;  // bleh
 
   // filestore
   bool filestore;
@@ -340,6 +367,7 @@ struct md_config_t {
   const char  *filestore_dev;
   bool filestore_btrfs_trans;
   bool filestore_btrfs_snap;
+  bool filestore_btrfs_clone_range;
   bool filestore_flusher;
   int filestore_flusher_max_fds;
   bool filestore_sync_flush;
@@ -369,6 +397,7 @@ struct md_config_t {
   int journal_max_write_entries;
   int journal_queue_max_ops;
   int journal_queue_max_bytes;
+  int journal_align_min_size;
 
   // block device
   bool  bdev_lock;
@@ -412,11 +441,10 @@ void env_to_deq(std::deque<const char*>& args);
 void argv_to_deq(int argc, const char **argv,
                  std::deque<const char*>& args);
 
-void parse_startup_config_options(std::vector<const char*>& args, bool isdaemon, const char *module_type);
+void parse_startup_config_options(std::vector<const char*>& args, const char *module_type);
 void parse_config_options(std::vector<const char*>& args);
 void parse_config_option_string(string& s);
 
-extern bool parse_ip_port(const char *s, entity_addr_t& addr, const char **end=0);
 extern bool parse_ip_port_vec(const char *s, vector<entity_addr_t>& vec);
 
 void generic_server_usage();
@@ -440,8 +468,14 @@ ExportControl *conf_get_export_control();
 #define CONF_SET_ARG_VAL(dest, type) \
 	conf_set_conf_val(dest, type, CONF_NEXT_VAL)
 
-#define CONF_SAFE_SET_ARG_VAL(dest, type) \
+#define CONF_VAL args[i]
+
+#define CONF_SAFE_SET_ARG_VAL_USAGE(dest, type, show_usage) \
 	do { \
+          __isarg = i+1 < args.size(); \
+          if (__isarg && !val_pos && \
+              args[i+1][0] == '-' && args[i+1][1] != '\0') \
+              __isarg = false; \
           if (type == OPT_BOOL) { \
 		if (val_pos) { \
 			CONF_SET_ARG_VAL(dest, type); \
@@ -449,10 +483,12 @@ ExportControl *conf_get_export_control();
 			conf_set_conf_val(dest, type, "true"); \
           } else if (__isarg || val_pos) { \
 		CONF_SET_ARG_VAL(dest, type); \
-	  } else if (args_usage) \
+	  } else if (show_usage && args_usage) \
 		args_usage(); \
 	} while (0)
 
+#define CONF_SAFE_SET_ARG_VAL(dest, type) CONF_SAFE_SET_ARG_VAL_USAGE(dest, type, true)
+
 #define CONF_SET_BOOL_ARG_VAL(dest) \
 	conf_set_conf_val(dest, OPT_BOOL, (val_pos ? &args[i][val_pos] : "true"))
 
