@@ -77,7 +77,9 @@ int main(int argc, const char **argv, const char *envp[])
   vector<const char*> args;
   argv_to_vec(argc, argv, args);
   env_to_vec(args);
-  common_init(args, "dumpjournal", false, false);
+
+  common_set_defaults(false);
+  common_init(args, "dumpjournal", false);
 
   vec_to_argv(args, argc, argv);
 
