@@ -12,8 +12,8 @@
  * 
  */
 
-#ifndef __ANCHOR_H
-#define __ANCHOR_H
+#ifndef CEPH_ANCHOR_H
+#define CEPH_ANCHOR_H
 
 #include <string>
 using std::string;
@@ -40,7 +40,7 @@ public:
   Anchor() : dn_hash(0), nref(0), updated(0) {}
   Anchor(inodeno_t i, inodeno_t di, __u32 hash, int nr, version_t u) :
     ino(i), dirino(di), dn_hash(hash), nref(nr), updated(u) { }
-  Anchor(inodeno_t i, inodeno_t di, const nstring &dname, int nr, version_t u) :
+  Anchor(inodeno_t i, inodeno_t di, const string &dname, int nr, version_t u) :
     ino(i), dirino(di),
     dn_hash(ceph_str_hash_linux(dname.data(), dname.length())),
     nref(nr), updated(u) { }
