@@ -755,7 +755,7 @@ public:
     reconnected_snaprealms[ino][client] = seq;
   }
   void process_imported_caps();
-  void process_reconnected_caps();
+  void choose_lock_states_and_reconnect_caps();
   void prepare_realm_split(SnapRealm *realm, client_t client, inodeno_t ino,
 			   map<client_t,MClientSnap*>& splits);
   void do_realm_invalidate_and_update_notify(CInode *in, int snapop, bool nosend=false);
@@ -879,6 +879,7 @@ public:
     if (in->get_parent_dn())
       touch_dentry(in->get_projected_parent_dn());
   }
+public:
   void touch_dentry(CDentry *dn) {
     // touch ancestors
     if (dn->get_dir()->get_inode()->get_projected_parent_dn())
@@ -890,7 +891,6 @@ public:
     else
       lru.lru_midtouch(dn);
   }
-public:
   void touch_dentry_bottom(CDentry *dn) {
     lru.lru_bottouch(dn);
   }
