@@ -12,13 +12,11 @@
  * 
  */
 
-#ifndef __MDS_EMETABLOB_H
-#define __MDS_EMETABLOB_H
+#ifndef CEPH_MDS_EMETABLOB_H
+#define CEPH_MDS_EMETABLOB_H
 
 #include <stdlib.h>
 
-#include "include/nstring.h"
-
 #include "../CInode.h"
 #include "../CDir.h"
 #include "../CDentry.h"
@@ -50,7 +48,7 @@ public:
   /* fullbit - a regular dentry + inode
    */
   struct fullbit {
-    nstring  dn;         // dentry
+    string  dn;         // dentry
     snapid_t dnfirst, dnlast;
     version_t dnv;
     inode_t inode;      // if it's not
@@ -62,7 +60,7 @@ public:
 
     bufferlist _enc;
 
-    fullbit(const nstring& d, snapid_t df, snapid_t dl, 
+    fullbit(const string& d, snapid_t df, snapid_t dl, 
 	    version_t v, inode_t& i, fragtree_t &dft, 
 	    map<string,bufferptr> &xa, const string& sym, bufferlist &sbl, bool dr) :
       //dn(d), dnfirst(df), dnlast(dl), dnv(v), 
@@ -135,7 +133,7 @@ public:
   /* remotebit - a dentry + remote inode link (i.e. just an ino)
    */
   struct remotebit {
-    nstring dn;
+    string dn;
     snapid_t dnfirst, dnlast;
     version_t dnv;
     inodeno_t ino;
@@ -144,7 +142,7 @@ public:
 
     bufferlist _enc;
 
-    remotebit(const nstring& d, snapid_t df, snapid_t dl, version_t v, inodeno_t i, unsigned char dt, bool dr) : 
+    remotebit(const string& d, snapid_t df, snapid_t dl, version_t v, inodeno_t i, unsigned char dt, bool dr) : 
       //dn(d), dnfirst(df), dnlast(dl), dnv(v), ino(i), d_type(dt), dirty(dr) { }
       _enc(256) {
       ::encode(d, _enc);
@@ -196,14 +194,14 @@ public:
    * nullbit - a null dentry
    */
   struct nullbit {
-    nstring dn;
+    string dn;
     snapid_t dnfirst, dnlast;
     version_t dnv;
     bool dirty;
 
     bufferlist _enc;
 
-    nullbit(const nstring& d, snapid_t df, snapid_t dl, version_t v, bool dr) : 
+    nullbit(const string& d, snapid_t df, snapid_t dl, version_t v, bool dr) : 
       //dn(d), dnfirst(df), dnlast(dl), dnv(v), dirty(dr) { }
       _enc(128) {
       ::encode(d, _enc);
@@ -479,16 +477,10 @@ private:
   void add_null_dentry(dirlump& lump, CDentry *dn, bool dirty) {
     // add the dir
     lump.nnull++;
-    if (dirty)
-      lump.get_dnull().push_front(nullbit(dn->get_name(), 
-					  dn->first, dn->last,
-					  dn->get_projected_version(), 
-					  dirty));
-    else
-      lump.get_dnull().push_back(nullbit(dn->get_name(), 
-					 dn->first, dn->last,
-					 dn->get_projected_version(), 
-					 dirty));
+    lump.get_dnull().push_back(nullbit(dn->get_name(), 
+				       dn->first, dn->last,
+				       dn->get_projected_version(), 
+				       dirty));
   }
 
   void add_remote_dentry(CDentry *dn, bool dirty) {
@@ -504,18 +496,11 @@ private:
       rdt = dn->get_projected_linkage()->get_remote_d_type();
     }
     lump.nremote++;
-    if (dirty)
-      lump.get_dremote().push_front(remotebit(dn->get_name(), 
-					      dn->first, dn->last,
-					      dn->get_projected_version(), 
-					      rino, rdt,
-					      dirty));
-    else
-      lump.get_dremote().push_back(remotebit(dn->get_name(), 
-					     dn->first, dn->last,
-					     dn->get_projected_version(), 
-					     rino, rdt,
-					     dirty));
+    lump.get_dremote().push_back(remotebit(dn->get_name(), 
+					   dn->first, dn->last,
+					   dn->get_projected_version(), 
+					   rino, rdt,
+					   dirty));
   }
 
   // return remote pointer to to-be-journaled inode
@@ -536,8 +521,10 @@ private:
     //cout << "journaling " << in->inode.ino << " at " << my_offset << std::endl;
 
     inode_t *pi = in->get_projected_inode();
-    if (!pdft) pdft = &in->dirfragtree;
-    if (!px) px = &in->xattrs;
+    if (!pdft)
+      pdft = &in->dirfragtree;
+    if (!px)
+      px = &in->xattrs;
 
     bufferlist snapbl;
     if (psnapbl)
@@ -546,25 +533,15 @@ private:
       in->encode_snap_blob(snapbl);
 
     lump.nfull++;
-    if (dirty) {
-      lump.get_dfull().push_front(fullbit(dn->get_name(), 
-					  dn->first, dn->last,
-					  dn->get_projected_version(), 
-					  *pi, *pdft, *px,
-					  in->symlink, snapbl,
-					  dirty));
-      if (pi) lump.get_dfull().front().inode = *pi;
-      return &lump.get_dfull().front().inode;
-    } else {
-      lump.get_dfull().push_back(fullbit(dn->get_name(), 
-					 dn->first, dn->last,
-					 dn->get_projected_version(),
-					  *pi, *pdft, *px,
-					 in->symlink, snapbl,
-					 dirty));
-      if (pi) lump.get_dfull().back().inode = *pi;
-      return &lump.get_dfull().back().inode;
-    }
+    lump.get_dfull().push_back(fullbit(dn->get_name(), 
+				       dn->first, dn->last,
+				       dn->get_projected_version(), 
+				       *pi, *pdft, *px,
+				       in->symlink, snapbl,
+				       dirty));
+    if (pi)
+      lump.get_dfull().back().inode = *pi;
+    return &lump.get_dfull().back().inode;
   }
 
   // convenience: primary or remote?  figure it out.
@@ -600,7 +577,7 @@ private:
     else
       in->encode_snap_blob(snapbl);
 
-    nstring empty;
+    string empty;
     delete root;
     root = new fullbit(empty,
 		       in->first, in->last,
