@@ -1,7 +1,7 @@
 // -*- mode:C++; tab-width:8; c-basic-offset:2; indent-tabs-mode:t -*- 
 // vim: ts=8 sw=2 smarttab
-#ifndef __MDSTYPES_H
-#define __MDSTYPES_H
+#ifndef CEPH_MDSTYPES_H
+#define CEPH_MDSTYPES_H
 
 
 #include <math.h>
@@ -16,14 +16,13 @@ using namespace std;
 
 #include "include/frag.h"
 #include "include/xlist.h"
-#include "include/nstring.h"
 
 #include <boost/pool/pool.hpp>
 
 #define CEPH_FS_ONDISK_MAGIC "ceph fs volume v011"
 
 
-//#define MDS_REF_SET      // define me for improved debug output, sanity checking
+#define MDS_REF_SET      // define me for improved debug output, sanity checking
 //#define MDS_AUTHPIN_SET  // define me for debugging auth pin leaks
 //#define MDS_VERIFY_FRAGSTAT    // do do (slow) sanity checking on frags
 
@@ -157,10 +156,15 @@ inline bool operator==(const frag_info_t &l, const frag_info_t &r) {
 }
 
 inline ostream& operator<<(ostream &out, const frag_info_t &f) {
-  return out << "f(v" << f.version
-	     << " m" << f.mtime
-	     << " " << f.size() << "=" << f.nfiles << "+" << f.nsubdirs
-	     << ")";    
+  if (f == frag_info_t())
+    return out << "f()";
+  out << "f(v" << f.version;
+  if (f.mtime != utime_t())
+    out << " m" << f.mtime;
+  if (f.nfiles || f.nsubdirs)
+    out << " " << f.size() << "=" << f.nfiles << "+" << f.nsubdirs;
+  out << ")";    
+  return out;
 }
 
 struct nest_info_t {
@@ -242,13 +246,21 @@ inline bool operator==(const nest_info_t &l, const nest_info_t &r) {
 }
 
 inline ostream& operator<<(ostream &out, const nest_info_t &n) {
-  return out << "n(v" << n.version
-	     << " rc" << n.rctime
-	     << " b" << n.rbytes
-	     << " a" << n.ranchors
-	     << " sr" << n.rsnaprealms
-	     << " " << n.rsize() << "=" << n.rfiles << "+" << n.rsubdirs
-	     << ")";    
+  if (n == nest_info_t())
+    return out << "n()";
+  out << "n(v" << n.version;
+  if (n.rctime != utime_t())
+    out << " rc" << n.rctime;
+  if (n.rbytes)
+    out << " b" << n.rbytes;
+  if (n.ranchors)
+    out << " a" << n.ranchors;
+  if (n.rsnaprealms)
+    out << " sr" << n.rsnaprealms;
+  if (n.rfiles || n.rsubdirs)
+    out << " " << n.rsize() << "=" << n.rfiles << "+" << n.rsubdirs;
+  out << ")";    
+  return out;
 }
 
 struct vinodeno_t {
@@ -379,7 +391,7 @@ struct inode_t {
 
   bool is_truncating() const { return (truncate_pending > 0); }
 
-  int64_t get_layout_size_increment() {
+  uint64_t get_layout_size_increment() {
     return layout.fl_object_size * layout.fl_stripe_count;
   }
 
@@ -585,15 +597,15 @@ struct dentry_key_t {
     bl.append("_", 1);
     bl.append(b);
   }
-  static void decode_helper(bufferlist::iterator& bl, nstring& nm, snapid_t& sn) {
-    nstring foo;
+  static void decode_helper(bufferlist::iterator& bl, string& nm, snapid_t& sn) {
+    string foo;
     ::decode(foo, bl);
 
     int i = foo.length()-1;
     while (foo[i] != '_' && i)
       i--;
     assert(i);
-    if (i+5 == foo.length() &&
+    if (i+5 == (int)foo.length() &&
 	foo[i+1] == 'h' &&
 	foo[i+2] == 'e' &&
 	foo[i+3] == 'a' &&
@@ -606,7 +618,7 @@ struct dentry_key_t {
       sscanf(foo.c_str() + i + 1, "%llx", &x);
       sn = x;
     }  
-    nm = nstring(i, foo.c_str());
+    nm = string(foo.c_str(), i);
   }
 };
 
@@ -627,14 +639,13 @@ inline bool operator<(const dentry_key_t& k1, const dentry_key_t& k2)
 
 
 /*
- * string_snap_t is a simple (nstring, snapid_t) pair
+ * string_snap_t is a simple (string, snapid_t) pair
  */
 struct string_snap_t {
-  nstring name;
+  string name;
   snapid_t snapid;
   string_snap_t() {}
   string_snap_t(const string& n, snapid_t s) : name(n), snapid(s) {}
-  string_snap_t(const nstring& n, snapid_t s) : name(n), snapid(s) {}
   string_snap_t(const char *n, snapid_t s) : name(n), snapid(s) {}
   void encode(bufferlist& bl) const {
     __u8 struct_v = 1;
@@ -1039,7 +1050,6 @@ class MDSCacheObject;
  */
 struct ClientLease {
   client_t client;
-  int mask;                 // CEPH_STAT_MASK_*
   MDSCacheObject *parent;
 
   ceph_seq_t seq;
@@ -1048,7 +1058,7 @@ struct ClientLease {
   xlist<ClientLease*>::item item_lease;         // global list
 
   ClientLease(client_t c, MDSCacheObject *p) : 
-    client(c), mask(0), parent(p), seq(0),
+    client(c), parent(p), seq(0),
     item_session_lease(this),
     item_lease(this) { }
 };
@@ -1068,7 +1078,7 @@ class MDSCacheObjectInfo {
 public:
   inodeno_t ino;
   dirfrag_t dirfrag;
-  nstring dname;
+  string dname;
   snapid_t snapid;
 
   MDSCacheObjectInfo() : ino(0) {}
