@@ -12,8 +12,8 @@
  * 
  */
 
-#ifndef __MEXPORTDIRNOTIFYACK_H
-#define __MEXPORTDIRNOTIFYACK_H
+#ifndef CEPH_MEXPORTDIRNOTIFYACK_H
+#define CEPH_MEXPORTDIRNOTIFYACK_H
 
 #include "msg/Message.h"
 #include <string>
@@ -30,6 +30,10 @@ class MExportDirNotifyAck : public Message {
     Message(MSG_MDS_EXPORTDIRNOTIFYACK) {
     this->dirfrag = dirfrag;
   }
+private:
+  ~MExportDirNotifyAck() {}
+
+public:
   const char *get_type_name() { return "ExNotA"; }
   void print(ostream& o) {
     o << "export_notify_ack(" << dirfrag << ")";
