@@ -13,8 +13,8 @@
  */
 
 
-#ifndef __MMDSSLAVEREQUEST_H
-#define __MMDSSLAVEREQUEST_H
+#ifndef CEPH_MMDSSLAVEREQUEST_H
+#define CEPH_MMDSSLAVEREQUEST_H
 
 #include "msg/Message.h"
 #include "mds/mdstypes.h"
@@ -106,6 +106,10 @@ public:
   MMDSSlaveRequest(metareqid_t ri, int o) : 
     Message(MSG_MDS_SLAVE_REQUEST),
     reqid(ri), op(o) { }
+private:
+  ~MMDSSlaveRequest() {}
+
+public:
   void encode_payload() {
     ::encode(reqid, payload);
     ::encode(op, payload);
