@@ -91,6 +91,7 @@ void Paxos::collect(version_t oldpn)
     
     MMonPaxos *collect = new MMonPaxos(mon->get_epoch(), MMonPaxos::OP_COLLECT, machine_id);
     collect->last_committed = last_committed;
+    collect->first_committed = first_committed;
     collect->pn = accepted_pn;
     mon->messenger->send_message(collect, mon->monmap->get_inst(*p));
   }
@@ -114,19 +115,8 @@ void Paxos::handle_collect(MMonPaxos *collect)
   // reply
   MMonPaxos *last = new MMonPaxos(mon->get_epoch(), MMonPaxos::OP_LAST, machine_id);
   last->last_committed = last_committed;
+  last->first_committed = first_committed;
   
-  // do we have an accepted but uncommitted value?
-  //  (it'll be at last_committed+1)
-  bufferlist bl;
-  if (mon->store->exists_bl_sn(machine_name, last_committed+1)) {
-    mon->store->get_bl_sn(bl, machine_name, last_committed+1);
-    assert(bl.length() > 0);
-    dout(10) << " sharing our accepted but uncommitted value for " << last_committed+1 
-	     << " (" << bl.length() << " bytes)" << dendl;
-    last->values[last_committed+1] = bl;
-    last->uncommitted_pn = accepted_pn;
-  }
-
   // can we accept this pn?
   if (collect->pn > accepted_pn) {
     // ok, accept it
@@ -143,38 +133,93 @@ void Paxos::handle_collect(MMonPaxos *collect)
   last->pn = accepted_pn;
   last->pn_from = accepted_pn_from;
 
-  // and share whatever data we have
-  if (collect->last_committed < last_committed) {
-    bufferlist bl;
-    version_t l = get_latest(bl);
-    assert(l <= last_committed);
+  // share whatever committed values we have
+  if (collect->last_committed < last_committed)
+    share_state(last, collect->first_committed, collect->last_committed);
 
-    version_t v = collect->last_committed;
-
-    // start with a stashed full copy?
-    /* hmm.
-    if (l > v + 10) {
-      last->latest_value.claim(bl);
-      last->latest_version = l;
-      v = l;
-    }
-    */
-
-    // include (remaining) incrementals
-    for (v++;
-	 v <= last_committed;
-	 v++) {
-      if (mon->store->exists_bl_sn(machine_name, v)) {
-	mon->store->get_bl_sn(last->values[v], machine_name, v);
-	dout(10) << " sharing " << v << " (" 
-		 << last->values[v].length() << " bytes)" << dendl;
-      }
-    }
+  // do we have an accepted but uncommitted value?
+  //  (it'll be at last_committed+1)
+  bufferlist bl;
+  if (mon->store->exists_bl_sn(machine_name, last_committed+1)) {
+    mon->store->get_bl_sn(bl, machine_name, last_committed+1);
+    assert(bl.length() > 0);
+    dout(10) << " sharing our accepted but uncommitted value for " << last_committed+1 
+	     << " (" << bl.length() << " bytes)" << dendl;
+    last->values[last_committed+1] = bl;
+    last->uncommitted_pn = accepted_pn;
   }
 
   // send reply
   mon->messenger->send_message(last, collect->get_source_inst());
-  delete collect;
+  collect->put();
+}
+
+void Paxos::share_state(MMonPaxos *m, version_t peer_first_committed, version_t peer_last_committed)
+{
+  assert(peer_last_committed < last_committed);
+
+  dout(10) << "share_state peer has fc " << peer_first_committed << " lc " << peer_last_committed << dendl;
+  version_t v = peer_last_committed;
+    
+  // start with a stashed full copy?
+  if (peer_last_committed < first_committed) {
+    bufferlist bl;
+    version_t l = get_latest(bl);
+    assert(l <= last_committed);
+    dout(10) << "share_state starting with latest " << l << " (" << bl.length() << " bytes)" << dendl;
+    m->latest_value.claim(bl);
+    m->latest_version = l;
+    v = l;
+  }
+
+  // include (remaining) incrementals
+  for (v++;
+       v <= last_committed;
+       v++) {
+    if (mon->store->exists_bl_sn(machine_name, v)) {
+      mon->store->get_bl_sn(m->values[v], machine_name, v);
+      dout(10) << " sharing " << v << " (" 
+	       << m->values[v].length() << " bytes)" << dendl;
+    }
+  }
+
+  m->last_committed = last_committed;
+}
+
+void Paxos::store_state(MMonPaxos *m)
+{
+  bool big_sync = m->values.size() > 5;
+
+  // stash?
+  if (m->latest_version && m->latest_version > last_committed) {
+    dout(10) << "store_state got stash version " << m->latest_version << ", zapping old states" << dendl;
+    stash_latest(m->latest_version, m->latest_value);
+
+    while (first_committed <= last_committed) {
+      dout(10) << "store_state trim " << first_committed << dendl;
+      mon->store->erase_sn(machine_name, first_committed);
+      first_committed++;
+    }
+    last_committed = m->latest_version;
+    first_committed = last_committed;
+    mon->store->put_int(first_committed, machine_name, "first_committed");
+  }
+
+  for (map<version_t,bufferlist>::iterator p = m->values.begin();
+       p != m->values.end() && p->first <= m->last_committed;
+       ++p) {
+    if (p->first <= last_committed)
+      continue;
+    if (p->first > last_committed + 1)
+      break;
+    last_committed = p->first;
+    dout(10) << "store_state got " << last_committed << " (" << p->second.length() << " bytes)" << dendl;
+    mon->store->put_bl_sn(p->second, machine_name, last_committed, !big_sync);
+  }
+
+  mon->store->put_int(last_committed, machine_name, "last_committed");
+  if (big_sync)
+    mon->store->sync();
 }
 
 
@@ -185,7 +230,7 @@ void Paxos::handle_last(MMonPaxos *last)
 
   if (!mon->is_leader()) {
     dout(10) << "not leader, dropping" << dendl;
-    delete last;
+    last->put();
     return;
   }
 
@@ -194,40 +239,12 @@ void Paxos::handle_last(MMonPaxos *last)
     // share committed values
     dout(10) << "sending commit to " << last->get_source() << dendl;
     MMonPaxos *commit = new MMonPaxos(mon->get_epoch(), MMonPaxos::OP_COMMIT, machine_id);
-    for (version_t v = last->last_committed+1;
-	 v <= last_committed;
-	 v++) {
-      mon->store->get_bl_sn(commit->values[v], machine_name, v);
-      dout(10) << " sharing " << v << " (" 
-	       << commit->values[v].length() << " bytes)" << dendl;
-    }
-    commit->last_committed = last_committed;
+    share_state(commit, last->first_committed, last->last_committed);
     mon->messenger->send_message(commit, last->get_source_inst());
   }
 
   // did we receive a committed value?
-  if (last->last_committed > last_committed) {
-    /* hmm.
-    if (last->latest_version) {
-      last_committed = last->latest_value;
-      dout(10) << "stashing latest full value " << last_committed << dendl;
-      stash_latest(last_committed, last->latest_value);
-    }
-    */
-    bool big_sync = last->last_committed - last_committed > 5;
-    for (version_t v = last_committed+1;
-	 v <= last->last_committed;
-	 v++) {
-      mon->store->put_bl_sn(last->values[v], machine_name, v, !big_sync);
-      dout(10) << "committing " << v << " " 
-	       << last->values[v].length() << " bytes" << dendl;
-    }
-    if (big_sync)
-      mon->store->sync();
-    last_committed = last->last_committed;
-    mon->store->put_int(last_committed, machine_name, "last_committed");
-    dout(10) << "last_committed now " << last_committed << dendl;
-  }
+  store_state(last);
       
   // do they accept your pn?
   if (last->pn > accepted_pn) {
@@ -287,7 +304,7 @@ void Paxos::handle_last(MMonPaxos *last)
     dout(10) << "old pn, ignoring" << dendl;
   }
 
-  delete last;
+  last->put();
 }
 
 void Paxos::collect_timeout()
@@ -363,7 +380,7 @@ void Paxos::handle_begin(MMonPaxos *begin)
   // can we accept this?
   if (begin->pn < accepted_pn) {
     dout(10) << " we accepted a higher pn " << accepted_pn << ", ignoring" << dendl;
-    delete begin;
+    begin->put();
     return;
   }
   assert(begin->pn == accepted_pn);
@@ -384,7 +401,7 @@ void Paxos::handle_begin(MMonPaxos *begin)
   accept->last_committed = last_committed;
   mon->messenger->send_message(accept, begin->get_source_inst());
   
-  delete begin;
+  begin->put();
 }
 
 // leader
@@ -396,13 +413,13 @@ void Paxos::handle_accept(MMonPaxos *accept)
   if (accept->pn != accepted_pn) {
     // we accepted a higher pn, from some other leader
     dout(10) << " we accepted a higher pn " << accepted_pn << ", ignoring" << dendl;
-    delete accept;
+    accept->put();
     return;
   }
   if (last_committed > 0 &&
       accept->last_committed < last_committed-1) {
     dout(10) << " this is from an old round, ignoring" << dendl;
-    delete accept;
+    accept->put();
     return;
   }
   assert(accept->last_committed == last_committed ||   // not committed
@@ -438,6 +455,7 @@ void Paxos::handle_accept(MMonPaxos *accept)
     finish_contexts(waiting_for_readable);
     finish_contexts(waiting_for_writeable);
   }
+  accept->put();
 }
 
 void Paxos::accept_timeout()
@@ -472,6 +490,7 @@ void Paxos::commit()
     MMonPaxos *commit = new MMonPaxos(mon->get_epoch(), MMonPaxos::OP_COMMIT, machine_id);
     commit->values[last_committed] = new_value;
     commit->pn = accepted_pn;
+    commit->last_committed = last_committed;
     
     mon->messenger->send_message(commit, mon->monmap->get_inst(*p));
   }
@@ -481,6 +500,8 @@ void Paxos::commit()
 }
 
 
+
+
 void Paxos::handle_commit(MMonPaxos *commit)
 {
   dout(10) << "handle_commit on " << commit->last_committed << dendl;
@@ -488,27 +509,13 @@ void Paxos::handle_commit(MMonPaxos *commit)
   if (!mon->is_peon()) {
     dout(10) << "not a peon, dropping" << dendl;
     assert(0);
-    delete commit;
+    commit->put();
     return;
   }
 
-  // commit locally.
-  bool big_sync = commit->values.size() > 2;
-  for (map<version_t,bufferlist>::iterator p = commit->values.begin();
-       p != commit->values.end();
-       ++p) {
-    assert(p->first <= last_committed+1);
-    if (p->first == last_committed+1) {
-      last_committed = p->first;
-      dout(10) << " storing " << last_committed << " (" << p->second.length() << " bytes)" << dendl;
-      mon->store->put_bl_sn(p->second, machine_name, last_committed, !big_sync);
-    }
-  }
-  if (big_sync)
-    mon->store->sync();
-  mon->store->put_int(last_committed, machine_name, "last_committed");
+  store_state(commit);
   
-  delete commit;
+  commit->put();
 
   finish_contexts(waiting_for_commit);
 }
@@ -532,9 +539,8 @@ void Paxos::extend_lease()
     if (*p == whoami) continue;
     MMonPaxos *lease = new MMonPaxos(mon->get_epoch(), MMonPaxos::OP_LEASE, machine_id);
     lease->last_committed = last_committed;
-    lease->lease_expire = lease_expire;
-    if (mon->is_full_quorum())
-      lease->first_committed = first_committed;
+    lease->lease_timestamp = lease_expire;
+    lease->first_committed = first_committed;
     mon->messenger->send_message(lease, mon->monmap->get_inst(*p));
   }
 
@@ -561,13 +567,23 @@ void Paxos::handle_lease(MMonPaxos *lease)
   if (!mon->is_peon() ||
       last_committed != lease->last_committed) {
     dout(10) << "handle_lease i'm not a peon, or they're not the leader, or the last_committed doesn't match, dropping" << dendl;
-    delete lease;
+    lease->put();
     return;
   }
 
+  if (lease->sent_timestamp > g_clock.now() + g_conf.mon_lease_wiggle_room) {
+    stringstream ss;
+    ss << "lease_expire from mon" << lease->get_source().num()
+       << " was sent from future time " << lease->sent_timestamp
+       << ", clocks not synchronized"
+       << std::endl;
+    mon->get_logclient()->log(LOG_WARN, ss);
+  }
+
   // extend lease
-  if (lease_expire < lease->lease_expire) 
-    lease_expire = lease->lease_expire;
+  if (lease_expire < lease->lease_timestamp) {
+    lease_expire = lease->lease_timestamp;
+  }
   
   state = STATE_ACTIVE;
   
@@ -578,7 +594,7 @@ void Paxos::handle_lease(MMonPaxos *lease)
   MMonPaxos *ack = new MMonPaxos(mon->get_epoch(), MMonPaxos::OP_LEASE_ACK, machine_id);
   ack->last_committed = last_committed;
   ack->first_committed = first_committed;
-  ack->lease_expire = lease_expire;
+  ack->lease_timestamp = g_clock.now();
   mon->messenger->send_message(ack, lease->get_source_inst());
 
   // (re)set timeout event.
@@ -595,7 +611,7 @@ void Paxos::handle_lease(MMonPaxos *lease)
   if (is_readable())
     finish_contexts(waiting_for_readable);
 
-  delete lease;
+  lease->put();
 }
 
 void Paxos::handle_lease_ack(MMonPaxos *ack)
@@ -624,8 +640,15 @@ void Paxos::handle_lease_ack(MMonPaxos *ack)
     dout(10) << "handle_lease_ack from " << ack->get_source() 
 	     << " dup (lagging!), ignoring" << dendl;
   }
-  
-  delete ack;
+
+  if (ack->sent_timestamp > g_clock.now() + g_conf.mon_lease_wiggle_room) {
+    stringstream ss;
+    ss << "lease_ack from mon" << from
+       << " was sent from future time " << ack->sent_timestamp
+       << ", clocks not synchronized." << std::endl;
+    mon->get_logclient()->log(LOG_WARN, ss);
+  }
+  ack->put();
 }
 
 void Paxos::lease_ack_timeout()
@@ -769,7 +792,7 @@ void Paxos::dispatch(PaxosServiceMessage *m)
   // election in progress?
   if (mon->is_starting()) {
     dout(5) << "election in progress, dropping " << *m << dendl;
-    delete m;
+    m->put();
     return;    
   }
 
@@ -860,7 +883,8 @@ void Paxos::update_observers()
     }
     ++iter;
     
-    if (observer->last_version == 0) {
+    if (observer->last_version == 0 ||
+	observer->last_version < first_committed) {
       ver = get_latest(bl);
       if (ver) {
 	dout(10) << " sending summary state v" << ver << " to " << observer->inst << dendl;
