@@ -98,8 +98,6 @@ using namespace std;
 #include "messages/MExportDirCancel.h"
 #include "messages/MExportDirPrep.h"
 #include "messages/MExportDirPrepAck.h"
-#include "messages/MExportDirWarning.h"
-#include "messages/MExportDirWarningAck.h"
 #include "messages/MExportDir.h"
 #include "messages/MExportDirAck.h"
 #include "messages/MExportDirNotify.h"
@@ -124,6 +122,7 @@ using namespace std;
 #include "messages/MLock.h"
 
 #include "messages/MClass.h"
+#include "messages/MClassAck.h"
 
 #include "config.h"
 
@@ -150,7 +149,6 @@ Message *decode_message(ceph_msg_header& header, ceph_msg_footer& footer,
   if (!g_conf.ms_nocrc) {
     __u32 front_crc = front.crc32c(0);
     __u32 middle_crc = middle.crc32c(0);
-    __u32 data_crc = data.crc32c(0);
 
     if (front_crc != footer.front_crc) {
       dout(0) << "bad crc in front " << front_crc << " != exp " << footer.front_crc << dendl;
@@ -166,13 +164,16 @@ Message *decode_message(ceph_msg_header& header, ceph_msg_footer& footer,
       *_dout << dendl;
       return 0;
     }
-    if (data_crc != footer.data_crc &&
-        !(footer.flags & CEPH_MSG_FOOTER_NOCRC)) {
-      dout(0) << "bad crc in data " << data_crc << " != exp " << footer.data_crc << dendl;
-      dout(20);
-      data.hexdump(*_dout);
-      *_dout << dendl;
-      return 0;
+
+    if ((footer.flags & CEPH_MSG_FOOTER_NOCRC) == 0) {
+      __u32 data_crc = data.crc32c(0);
+      if (data_crc != footer.data_crc) {
+	dout(0) << "bad crc in data " << data_crc << " != exp " << footer.data_crc << dendl;
+	dout(20);
+	data.hexdump(*_dout);
+	*_dout << dendl;
+	return 0;
+      }
     }
   }
 
@@ -202,10 +203,10 @@ Message *decode_message(ceph_msg_header& header, ceph_msg_footer& footer,
   case MSG_GETPOOLSTATSREPLY:
     m = new MGetPoolStatsReply;
     break;
-  case MSG_POOLOP:
+  case CEPH_MSG_POOLOP:
     m = new MPoolOp;
     break;
-  case MSG_POOLOPREPLY:
+  case CEPH_MSG_POOLOP_REPLY:
     m = new MPoolOpReply;
     break;
   case MSG_MON_COMMAND:
@@ -441,14 +442,6 @@ Message *decode_message(ceph_msg_header& header, ceph_msg_footer& footer,
     m = new MExportDirPrepAck();
     break;
 
-  case MSG_MDS_EXPORTDIRWARNING:
-    m = new MExportDirWarning;
-    break;
-  case MSG_MDS_EXPORTDIRWARNINGACK:
-    m = new MExportDirWarningAck;
-    break;
-
-    
   case MSG_MDS_EXPORTCAPS:
     m = new MExportCaps;
     break;
@@ -500,6 +493,10 @@ Message *decode_message(ceph_msg_header& header, ceph_msg_footer& footer,
     m = new MClass();
     break;
 
+  case MSG_CLASS_ACK:
+    m = new MClassAck();
+    break;
+
   default:
     dout(0) << "can't decode unknown message type " << type << " MSG_AUTH=" << CEPH_MSG_AUTH << dendl;
     if (g_conf.ms_die_on_bad_msg)
@@ -517,7 +514,9 @@ Message *decode_message(ceph_msg_header& header, ceph_msg_footer& footer,
     m->decode_payload();
   }
   catch (buffer::error *e) {
-    dout(0) << "failed to decode message of type " << type << ": " << *e << dendl;
+    dout(0) << "failed to decode message of type " << type
+	    << " v" << header.version
+	    << ": " << *e << dendl;
     delete e;
     if (g_conf.ms_die_on_bad_msg)
       assert(0);
