@@ -12,8 +12,8 @@
  * 
  */
 
-#ifndef __MESSAGE_H
-#define __MESSAGE_H
+#ifndef CEPH_MESSAGE_H
+#define CEPH_MESSAGE_H
  
 /* public message types */
 #include "include/types.h"
@@ -38,8 +38,8 @@
 
 #define MSG_MON_GLOBAL_ID          60
 
-#define MSG_POOLOP                 49
-#define MSG_POOLOPREPLY            48
+// #define MSG_POOLOP                 49
+// #define MSG_POOLOPREPLY            48
 
 #define MSG_ROUTE                  47
 #define MSG_FORWARD                46
@@ -93,7 +93,6 @@
 #define MSG_MDS_DENTRYUNLINK       0x208
 #define MSG_MDS_FRAGMENTNOTIFY     0x209
 #define MSG_MDS_OFFLOAD_TARGETS    0x20a
-#define MSG_MDS_OFFLOAD_COMPLETE   0x20b
 #define MSG_MDS_DENTRYLINK         0x20c
 
 #define MSG_MDS_LOCK               0x300
@@ -130,6 +129,7 @@ using std::list;
 
 #include "include/types.h"
 #include "include/buffer.h"
+#include "common/Throttle.h"
 #include "msg_types.h"
 
 #include "common/debug.h"
@@ -146,12 +146,12 @@ struct RefCountedObject {
   virtual ~RefCountedObject() {}
   
   RefCountedObject *get() {
-    //generic_dout(0) << "RefCountedObject::get " << this << " " << nref.test() << " -> " << (nref.test() + 1) << dendl;
+    //generic_dout(0) << "RefCountedObject::get " << this << " " << nref.read() << " -> " << (nref.read() + 1) << dendl;
     nref.inc();
     return this;
   }
   void put() {
-    //generic_dout(0) << "RefCountedObject::put " << this << " " << nref.test() << " -> " << (nref.test() - 1) << dendl;
+    //generic_dout(0) << "RefCountedObject::put " << this << " " << nref.read() << " -> " << (nref.read() - 1) << dendl;
     if (nref.dec() == 0)
       delete this;
   }
@@ -164,15 +164,18 @@ struct Connection : public RefCountedObject {
   int peer_type;
   entity_addr_t peer_addr;
   unsigned features;
+  RefCountedObject *pipe;
 
 public:
-  Connection() : nref(1), lock("Connection::lock"), priv(NULL), peer_type(-1), features(0) {}
+  Connection() : nref(1), lock("Connection::lock"), priv(NULL), peer_type(-1), features(0), pipe(NULL) {}
   ~Connection() {
     //generic_dout(0) << "~Connection " << this << dendl;
     if (priv) {
       //generic_dout(0) << "~Connection " << this << " dropping priv " << priv << dendl;
       priv->put();
     }
+    if (pipe)
+      pipe->put();
   }
 
   Connection *get() {
@@ -197,6 +200,20 @@ public:
     return NULL;
   }
 
+  RefCountedObject *get_pipe() {
+    Mutex::Locker l(lock);
+    if (pipe)
+      return pipe->get();
+    return NULL;
+  }
+  void clear_pipe() {
+    Mutex::Locker l(lock);
+    if (pipe) {
+      pipe->put();
+      pipe = NULL;
+    }
+  }
+
   int get_peer_type() { return peer_type; }
   void set_peer_type(int t) { peer_type = t; }
   
@@ -229,32 +246,45 @@ protected:
   utime_t recv_stamp;
   Connection *connection;
 
+  // release our size in bytes back to this throttler when our payload
+  // is adjusted or when we are destroyed.
+  Throttle *throttler;
+
+  // keep track of how big this message was when we reserved space in
+  // the msgr dispatch_throttler, so that we can properly release it
+  // later.  this is necessary because messages can enter the dispatch
+  // queue locally (not via read_message()), and those are not
+  // currently throttled.
+  uint64_t dispatch_throttle_size;
+
   friend class Messenger;
 
-  bool _forwarded;
-  entity_inst_t _orig_source_inst;
-  
 public:
   atomic_t nref;
 
-  Message() : connection(NULL), _forwarded(false), nref(0) {
+  Message() : connection(NULL), dispatch_throttle_size(0), nref(1) {
     memset(&header, 0, sizeof(header));
     memset(&footer, 0, sizeof(footer));
+    throttler = NULL;
   };
-  Message(int t) : connection(NULL), _forwarded(false), nref(0) {
+  Message(int t) : connection(NULL), dispatch_throttle_size(0), nref(1) {
     memset(&header, 0, sizeof(header));
     header.type = t;
     header.version = 1;
     header.priority = 0;  // undef
     header.data_off = 0;
     memset(&footer, 0, sizeof(footer));
+    throttler = NULL;
   }
+protected:
   virtual ~Message() { 
     assert(nref.read() == 0);
     if (connection)
       connection->put();
+    if (throttler)
+      throttler->put(payload.length() + middle.length() + data.length());
   }
-
+public:
   Message *get() {
     //int r = 
     nref.inc();
@@ -271,25 +301,65 @@ public:
   }
 
   Connection *get_connection() { return connection; }
-  void set_connection(Connection *c) { connection = c; }
+  void set_connection(Connection *c) {
+    if (connection)
+      connection->put();
+    connection = c;
+  }
+  void set_throttler(Throttle *t) { throttler = t; }
+  Throttle *get_throttler() { return throttler; }
  
+  void set_dispatch_throttle_size(uint64_t s) { dispatch_throttle_size = s; }
+  uint64_t get_dispatch_throttle_size() { return dispatch_throttle_size; }
+
   ceph_msg_header &get_header() { return header; }
   void set_header(const ceph_msg_header &e) { header = e; }
   void set_footer(const ceph_msg_footer &e) { footer = e; }
   ceph_msg_footer &get_footer() { return footer; }
 
-  void clear_payload() { payload.clear(); middle.clear(); }
+  /*
+   * If you use get_[data, middle, payload] you shouldn't
+   * use it to change those bufferlists unless you KNOW
+   * there is no throttle being used. The other
+   * functions are throttling-aware as appropriate.
+   */
+
+  void clear_payload() {
+    if (throttler) throttler->put(payload.length() + middle.length());
+    payload.clear();
+    middle.clear();
+  }
+  void clear_data() {
+    if (throttler) throttler->put(data.length());
+    data.clear();
+  }
+
   bool empty_payload() { return payload.length() == 0; }
   bufferlist& get_payload() { return payload; }
-  void set_payload(bufferlist& bl) { payload.claim(bl); }
-  void copy_payload(const bufferlist& bl) { payload = bl; }
+  void set_payload(bufferlist& bl) {
+    if (throttler) throttler->put(payload.length());
+    payload.claim(bl);
+    if (throttler) throttler->take(payload.length());
+  }
 
-  void set_middle(bufferlist& bl) { middle.claim(bl); }
+  void set_middle(bufferlist& bl) {
+    if (throttler) throttler->put(payload.length());
+    middle.claim(bl);
+    if (throttler) throttler->take(payload.length());
+  }
   bufferlist& get_middle() { return middle; }
 
-  void set_data(const bufferlist &d) { data = d; }
-  void copy_data(const bufferlist &d) { data = d; }
+  void set_data(const bufferlist &d) {
+    if (throttler) throttler->put(data.length());
+    data = d;
+    if (throttler) throttler->take(data.length());
+  }
+
   bufferlist& get_data() { return data; }
+  void claim_data(bufferlist& bl) {
+    if (throttler) throttler->put(data.length());
+    bl.claim(data);
+  }
   off_t get_data_len() { return data.length(); }
 
   void set_recv_stamp(utime_t t) { recv_stamp = t; }
@@ -335,8 +405,6 @@ public:
 
   // forwarded?
   entity_inst_t get_orig_source_inst() {
-    if (_forwarded)
-      return _orig_source_inst;
     return get_source_inst();
   }
   entity_name_t get_orig_source() {
@@ -345,10 +413,6 @@ public:
   entity_addr_t get_orig_source_addr() {
     return get_orig_source_inst().addr;
   }
-  void set_orig_source_inst(entity_inst_t& i) {
-    _forwarded = true;
-    _orig_source_inst = i;
-  }
 
   // virtual bits
   virtual void decode_payload() = 0;
@@ -365,6 +429,8 @@ extern Message *decode_message(ceph_msg_header &header, ceph_msg_footer& footer,
 			       bufferlist& front, bufferlist& middle, bufferlist& data);
 inline ostream& operator<<(ostream& out, Message& m) {
   m.print(out);
+  if (m.get_header().version)
+    out << " v" << m.get_header().version;
   return out;
 }
 
