@@ -436,7 +436,10 @@ int SimpleMessenger::send_message(Message *m, Connection *con)
   if (pipe) {
     submit_message(m, pipe);
     pipe->put();
-  } // else we raced with reaper()
+  } else {
+    // else we raced with reaper()
+    m->put();
+  }
   return 0;
 }
 
@@ -1255,8 +1258,10 @@ void SimpleMessenger::Pipe::discard_queue()
       (*r)->put();
   out_q.clear();
   for (map<int,list<Message*> >::iterator p = in_q.begin(); p != in_q.end(); p++)
-    for (list<Message*>::iterator r = p->second.begin(); r != p->second.end(); r++)
+    for (list<Message*>::iterator r = p->second.begin(); r != p->second.end(); r++) {
+      messenger->dispatch_throttle_release((*r)->get_dispatch_throttle_size());
       (*r)->put();
+    }
   in_q.clear();
   in_qlen = 0;
 }
@@ -2335,11 +2340,6 @@ bool SimpleMessenger::register_entity(entity_name_t name)
 
 void SimpleMessenger::submit_message(Message *m, Pipe *pipe)
 { 
-  if (!pipe) {
-    m->put();
-    return;
-  }
-
   lock.Lock();
   if (pipe == dispatch_queue.local_pipe) {
     dout(20) << "submit_message " << *m << " local" << dendl;
