@@ -68,44 +68,3 @@ int tcp_write(int sd, const char *buf, int len) {
   }
   return 0;
 }
-
-int tcp_hostlookup(char *str, sockaddr_in& ta)
-{
-  char *host = str;
-  char *port = 0;
-  
-  for (int i=0; str[i]; i++) {
-    if (str[i] == ':') {
-      port = str+i+1;
-      str[i] = 0;
-      break;
-    }
-  }
-  if (!port) {
-    cerr << "addr '" << str << "' doesn't look like 'host:port'" << std::endl;
-    return -1;
-  } 
-  //cout << "host '" << host << "' port '" << port << "'" << std::endl;
-
-  int iport = atoi(port);
-  
-  struct hostent *myhostname = gethostbyname( host ); 
-  if (!myhostname) {
-    cerr << "host " << host << " not found" << std::endl;
-    return -1;
-  }
-
-  memset(&ta, 0, sizeof(ta));
-
-  //cout << "addrtype " << myhostname->h_addrtype << " len " << myhostname->h_length << std::endl;
-
-  ta.sin_family = myhostname->h_addrtype;
-  memcpy((char *)&ta.sin_addr,
-         myhostname->h_addr, 
-         myhostname->h_length);
-  ta.sin_port = iport;
-    
-  cout << "lookup '" << host << ":" << port << "' -> " << ta << std::endl;
-
-  return 0;
-}
